@@ -83,12 +83,14 @@ zapsmall(data.frame(g),digits=15)
 pinvgauss(c(0,0.1,1,2.3,3.1,NA),mean=c(1,2,3,0,1,2),dispersion=0.5)
 pinvgauss(c(0,0.1,1,2.3,3.1,NA),mean=c(1,2,3,0,1,2),dispersion=0.5,log.p=TRUE)
 pinvgauss(c(0,0.1,1,2.3,3.1,NA),mean=c(1,2,3,0,1,2),dispersion=0.5,lower.tail=FALSE,log.p=TRUE)
+pinvgauss(1,mean=c(1,2,NA))
 p <- c(0,0.001,0.5,0.999,1)
 qinvgauss(p,mean=1.3,dispersion=0.6)
 qinvgauss(p,mean=1.3,dispersion=0.6,lower.tail=FALSE)
+qinvgauss(0.5,mean=c(1,2,NA))
 qinvgauss(log(p),mean=1.3,dispersion=0.6,log.p=TRUE)
 qinvgauss(log(p),mean=1.3,dispersion=0.6,lower.tail=FALSE,log.p=TRUE)
-
+rinvgauss(5,mean=c(1,NA,3,Inf,1e10),disp=c(2,3,NA,Inf,4))
 
 ### extra tests done only locally
 
